@@ -10,8 +10,15 @@ from wikibot.wikiuser import WikiUser
 from wikibot.wikiclient import WikiClient
 from wikibot.wikipush import WikiPush
 from ormigrate.fixer import PageFixer
+from ormigrate.toolbox import HelperFunctions as hf
+from wikifile.wikiFileManager import WikiFileManager
+from wikifile.wikiFile import WikiFile
+from wikifile.wikiRender import WikiRender
+import inspect
 import os
 import time
+import ntpath
+from pathlib import Path
 from openresearch.openresearch import OpenResearch
 
 from ormigrate.issue41 import AcronymLengthFixer
@@ -20,6 +27,34 @@ from ormigrate.issue71 import DateFixer
 from ormigrate.eventSeriesFixer import EventSeriesProvenanceFixer, EventSeriesTitleFixer
 from ormigrate.issue152 import AcceptanceRateFixer
 from ormigrate.issue170_curation import CurationQualityChecker
+from ormigrate.issue195 import BiblographicFieldFixer
+
+
+class OREntity(JSONAble):
+    def __init__(self):
+        '''
+        Constructor
+        '''
+
+
+    def fixRecord(self, record):
+        '''
+        fix my dict representation
+        '''
+        invalidKeys = []
+        for key in record.keys():
+            value = record[key]
+            if type(value) == list:
+                # TODO: handle properly e.g. by marking and converting list to
+                # comma separated list
+
+                invalidKeys.append(key)
+                print("invalid list %s=%s in %s" % (key, record[key], record))
+            if value is None:
+                invalidKeys.append(key)
+
+        for key in invalidKeys:
+            record.pop(key)
 
 class OREntityList(JSONAbleList):
     '''
@@ -32,18 +67,35 @@ class OREntityList(JSONAbleList):
         self.wikiClient=None
         self.wikiPush=None
         self.askExtra=""
-        
+
     def getList(self):
         return self.__dict__[self.listName]
-    
+
+    def updateEntity(self,entity,identifier='pageTitle'):
+        '''
+        Add/Update the given entity in the entityList
+        '''
+        if hasattr(entity,identifier):
+            attributes = [*entity.__dict__]
+            for origEntity in self.getList():
+                if origEntity.pageTitle == entity.pageTitle:
+                    origAttributes = [*origEntity.__dict__]
+                    difference = set(attributes) - set(origAttributes)
+                    for attr in difference:
+                        setattr(origEntity,attr,getattr(entity,attr))
+                    return
+            self.getList().append(entity)
+        else:
+            raise Exception('identifier not found in entity given')
+
     def getLookup(self,attrName:str,withDuplicates:bool=False):
         '''
         create a lookup dictionary by the given attribute name
-        
+
         Args:
             attrName(str): the attribute to lookup
             withDuplicates(bool): whether to retain single values or lists
-        
+
         Return:
             a dictionary for lookup
         '''
@@ -63,26 +115,26 @@ class OREntityList(JSONAbleList):
                         lookupResult=[entity]
                     else:
                         lookupResult=entity
-                lookup[value]=lookupResult  
+                lookup[value]=lookupResult
         if withDuplicates:
-            return lookup  
+            return lookup
         else:
             return lookup,duplicates
-    
+
     def getEntityName(self):
         '''
         get my entity name
         '''
         return self.clazz.__name__
-    
+
     def getAskQuery(self,askExtra="",propertyLookupList=None):
         '''
         get the query that will ask for all my events
-        
+
         Args:
            askExtra(str): any additional SMW ask query constraints
            propertyLookupList:  a list of dicts for propertyLookup
-           
+
         Return:
             str: the SMW ask query
         '''
@@ -102,7 +154,7 @@ class OREntityList(JSONAbleList):
             ask+="|?%s=%s\n" % (propName,name)
         ask+="}}"
         return ask
-    
+
     def getJsonFile(self):
         '''
         get the json File for me
@@ -112,11 +164,11 @@ class OREntityList(JSONAbleList):
         jsonPrefix="%s/%s" % (cachePath,self.getEntityName())
         jsonFilePath="%s.json" % jsonPrefix
         return jsonFilePath
-    
+
     def fromCache(self,wikiuser:WikiUser):
         '''
         Args:
-            wikiuser: the wikiuser to use 
+            wikiuser: the wikiuser to use
         '''
         jsonFilePath=self.getJsonFile()
         # TODO: fix upstream pyLodStorage
@@ -126,7 +178,12 @@ class OREntityList(JSONAbleList):
         else:
             self.fromWiki(wikiuser,askExtra=self.askExtra,profile=self.profile)
             self.storeToJsonFile(jsonPrefix)
-            
+
+    def toCache(self):
+        jsonFilePath = self.getJsonFile()
+        jsonPrefix = jsonFilePath.replace(".json", "")
+        self.storeToJsonFile(jsonPrefix)
+
     def fromWiki(self,wikiuser:WikiUser,askExtra="",profile=False):
         '''
         read me from a wiki using the givne WikiUser configuration
@@ -145,17 +202,17 @@ class OREntityList(JSONAbleList):
             print("query of %d %s records took %5.1f s" % (len(records),entityName,elapsed))
         self.fromLoD(records)
         return records
-    
+
     def fromSQLTable(self,sqlDB,entityInfo):
         lod=sqlDB.queryAll(entityInfo)
         self.fromLoD(lod)
-        
-    def fromLoD(self,lod): 
+
+    def fromLoD(self,lod):
         '''
         create me from the given list of dicts
         '''
-        errors=[] 
-        entityList=self.getList()  
+        errors=[]
+        entityList=self.getList()
         for record in lod:
             # call the constructor to get a new instance
             try:
@@ -175,14 +232,47 @@ class OREntityList(JSONAbleList):
                 if self.debug:
                     print(error)
         return errors
-    
+
+
+    def fromWikiSonBackupFiles(self,wikiSonName,backupdir=str(Path.home() / 'wikibackup'/ 'or' ),wikiId= 'or',listOfItems=[]):
+        """
+        Create me from the backup wiki files of OR Entity
+
+        """
+        filepaths = list(hf.absoluteFilePaths(backupdir))
+        wikiRender = WikiRender()
+        wikiFileList=[]
+        for file in filepaths:
+            filename = ntpath.basename(file).replace('.wiki','')
+            filepath = ntpath.dirname(file)
+            if len(listOfItems) > 0:
+                if filename in listOfItems:
+                    wikiFile = WikiFile(filename, filepath, wikiRender)
+                    wikiFileList.append(wikiFile)
+            else:
+                wikiFile = WikiFile(filename,filepath,wikiRender)
+                wikiFileList.append(wikiFile)
+        wikiFileManger= WikiFileManager(wikiId)
+        LOD=wikiFileManger.convertWikiFilesToLOD(wikiFileList,wikiSonName)
+        self.fromLoD(LOD)
+
+    def getLoD(self):
+        """
+        Return the LoD of the entities in the list
+        """
+        LoD= []
+        for entity in self.getList():
+            LoD.append(entity.__dict__)
+        return LoD
+
+
     def getRatedLod(self,ratingCallback=None):
         '''
         get the list of dicts with a potential rating
-        
+
         Args:
             ratingCallback(func): a function to be called for rating of this entity
-            
+
         Returns:
             list: a list of dicts with the rating
         '''
@@ -199,12 +289,12 @@ class OREntityList(JSONAbleList):
                 if hasattr(entity,name):
                     entityRecord[name]=getattr(entity,name)
             if ratingCallback is not None:
-                ratingErrors=ratingCallback(entity,entityRecord)   
+                ratingErrors=ratingCallback(entity,entityRecord)
                 if len(ratingErrors)>0:
-                    errors.append(ratingErrors) 
+                    errors.append(ratingErrors)
             lod.append(entityRecord)
         return lod,errors
-        
+
 class EventSeriesList(OREntityList):
     '''
     i represent a list of EventSeries
@@ -221,7 +311,7 @@ class EventSeriesList(OREntityList):
             { 'prop':'DblpSeries',  'name': 'dblpSeries' }
         ]
         
-class EventSeries(JSONAble):
+class EventSeries(OREntity):
     '''
     '''
     def __init__(self):
@@ -333,8 +423,10 @@ class EventList(OREntityList):
             { 'prop':'Accepted_papers',     'name': 'acceptedPapers'},
             { 'prop':'Submitted_papers',    'name': 'submittedPapers'}
         ]               
-    
-class Event(JSONAble):
+
+
+
+class Event(OREntity):
     '''
     I represent an Event
     
@@ -437,23 +529,7 @@ This CfP was obtained from [http://www.wikicfp.com/cfp/servlet/event.showcfp?eve
         
         return samplesWikiSon
     
-    def fixRecord(self,record):
-        '''
-        fix my dict representation
-        '''
-        invalidKeys=[]
-        for key in record.keys():
-            value=record[key]
-            if type(value)==list:
-                # TODO: handle properly e.g. by marking and converting list to 
-                # comma separated list
-                invalidKeys.append(key)
-                print ("invalid list %s=%s in %s"  % (key,record[key],record ))
-            if value is None:
-                invalidKeys.append(key)
-                
-        for key in invalidKeys:
-            record.pop(key)
+
             
     @classmethod       
     def rateMigration(cls,event,eventRecord):
@@ -480,6 +556,10 @@ This CfP was obtained from [http://www.wikicfp.com/cfp/servlet/event.showcfp?eve
             {
                 "column": "AcceptanceRatePainRating",
                 "fixer": AcceptanceRateFixer
+            },
+            {
+                "column": "BiblographicFieldFixer",
+                "fixer": BiblographicFieldFixer
             }
         ]
         return PageFixer.rateWithFixers(pageFixerList, event,eventRecord)
@@ -504,7 +584,7 @@ class CountryList(OREntityList):
         ]       
         
     def getDefault(self):
-        jsonFilePrefix="%s/countries" % OpenResearch.getResourcePath()
+        jsonFilePrefix="%s/orCountries" % OpenResearch.getResourcePath()
         self.restoreFromJsonFile(jsonFilePrefix)
 
     @classmethod
